==> Xorfilter-0.0.3.crate: Xorfilter-0.0.3/src/lib.rs <==
#[derive(Debug, Default, Clone)]
pub struct Xor8 {
	Seed: u64,
	BlockLength: u32,
	Fingerprints: Vec<u8> 
}

/*
impl Default for Xor8 {
    fn default () -> Xor8 {
        Xor8{Seed: 0, BlockLength: 0, Fingerprints: Vec::new()}
    }
}
*/

#[derive(Clone, Debug)]
struct xorset {
	xormask: u64,
	count: u32
}

#[derive(Debug)]
struct hashes {
	h:  u64,
	h0: u32,
	h1: u32,
	h2: u32
}

#[derive(Copy, Clone, Debug)]
struct keyindex {
	hash: u64,
	index: u32 
}

pub fn murmur64(mut h: u64) -> u64 {
	h ^= h >> 33;
	h = h.wrapping_mul(0xff51afd7ed558ccd);
	h ^= h >> 33;
	h = h.wrapping_mul(0xc4ceb9fe1a85ec53);
	h ^= h >> 33;
	h
}

pub fn splitmix64(mut seed: u64) -> u64 {
    let mut z = seed.wrapping_add(0x9e3779b97f4a7c15); 
    z = (z ^ (z >> 30)).wrapping_mul(0xbf58476d1ce4e5b9); 
    z = (z ^ (z >> 27)).wrapping_mul(0x94d049bb133111eb);  
    z ^ (z >> 31)
}

pub fn mixsplit(key: u64, seed: u64) -> u64 {
    murmur64(key.wrapping_add(seed)) 
}

pub fn rotl64(n: u64, c: i64) -> u64 {
    (n << (c & 63)) | (n >> ((-c) & 63))
}

// http://lemire.me/blog/2016/06/27/a-fast-alternative-to-the-modulo-reduction/
pub fn reduce(hash: u32, n: u32) -> u32 {
        ((hash as u64).wrapping_mul(n as u64) >> 32) as u32
}

pub fn fingerprint(hash: u64) -> u64 {
	hash ^ (hash >> 32)
}

#[derive(Debug, Default, Clone)]
pub struct Xorfilter {
    filter: Xor8
}

impl Xorfilter {
    
    // TODO
    // type so this works
    // beyond just Xor8
    pub fn new() -> Xorfilter { 
        
               Xorfilter { 
                    filter: Xor8{Seed: 0, BlockLength: 0, Fingerprints: Vec::new()}
               }
               
    }

    fn geth0h1h2(&mut self, k: u64) -> hashes {
        let mut hash = mixsplit(k, self.filter.Seed);
        let mut r0 = hash as u32;
        let mut r1 = rotl64(hash, 21) as u32;
        let mut r2 = rotl64(hash, 42) as u32;
        
        let answer = hashes {
            h: hash,
            h0: reduce(r0, self.filter.BlockLength),
            h1: reduce(r1, self.filter.BlockLength),
            h2: reduce(r2, self.filter.BlockLength),
        };
        
        answer
    }
    
    fn geth0(&mut self, hash: u64) -> u32 {
        let r0 = hash as u32;
        reduce(r0, self.filter.BlockLength) 
    }
    
    fn geth1(&mut self, hash: u64) -> u32 {
        let r1 = rotl64(hash, 21) as u32;
        reduce(r1, self.filter.BlockLength)
    }
    
    fn geth2(&mut self, hash: u64) -> u32 {
        let r2 = rotl64(hash, 42) as u32;
        reduce(r2, self.filter.BlockLength)
    }
    
    pub fn populate(&mut self, keys: Vec<u64>) { //-> Xor8 {
        let size = keys.len();
        let mut capacity = 32 + ((1.23*(size as f64)).ceil() as u32); //u32);
        capacity = capacity / 3 * 3; // round it down to a multiple of 3
        self.filter.BlockLength = capacity / 3; 
     
        self.filter.Fingerprints = vec![0; capacity as usize]; 
        let mut rngcounter: u64 = 1;
        self.filter.Seed = splitmix64(rngcounter);
        
        // unsure if defaulting values to zero will hurt performance
        // may need to change implementation in the future
        
        let mut Q0: Vec<keyindex> = vec![keyindex {hash: 0, index: 0}; self.filter.BlockLength as usize]; 
        let mut Q1: Vec<keyindex> = vec![keyindex {hash: 0, index: 0}; self.filter.BlockLength as usize]; 
        let mut Q2: Vec<keyindex> = vec![keyindex {hash: 0, index: 0}; self.filter.BlockLength as usize]; 
        
        let mut stack: Vec<keyindex> = vec![keyindex {hash: 0, index: 0}; size]; 
        let mut sets0: Vec<xorset> = vec![xorset {xormask:0, count:0}; self.filter.BlockLength as usize]; 
        let mut sets1: Vec<xorset> = vec![xorset {xormask:0, count:0}; self.filter.BlockLength as usize]; 
        let mut sets2: Vec<xorset> = vec![xorset {xormask:0, count:0}; self.filter.BlockLength as usize]; 
        
        while true {
            for i in 0..size {
                
                let key = keys[i];
                let hs = self.geth0h1h2(key);
                sets0[hs.h0 as usize].xormask ^= hs.h;
                sets0[hs.h0 as usize].count = sets0[hs.h0 as usize].count + 1;
                sets1[hs.h1 as usize].xormask ^= hs.h;
                sets1[hs.h1 as usize].count = sets1[hs.h1 as usize].count + 1;
                sets2[hs.h2 as usize].xormask ^= hs.h;
                sets2[hs.h2 as usize].count = sets2[hs.h2 as usize].count + 1;
                
	    }
                
            
            let mut Q0size = 0;
            let mut Q1size = 0;
            let mut Q2size = 0;
           
            for i in 0u32..self.filter.BlockLength {
                if sets0[i as usize].count == 1 {
                            Q0[Q0size].index = i;
                            Q0[Q0size].hash = sets0[i as usize].xormask;
                            Q0size = Q0size + 1;
                        }
            }
            
            for i in 0u32..self.filter.BlockLength {
			if sets1[i as usize].count == 1 {
				Q1[Q1size].index = i;
				Q1[Q1size].hash = sets1[i as usize].xormask;
				Q1size = Q1size + 1;
			}
		}
		
            for i in 0u32..self.filter.BlockLength {
			if sets2[i as usize].count == 1 {
				Q2[Q2size].index = i;
				Q2[Q2size].hash = sets2[i as usize].xormask;
				Q2size = Q2size + 1;
			}
		}
                
            let mut stacksize = 0;
               
            while Q0size+Q1size+Q2size > 0 {
                while Q0size > 0 {
                    Q0size = Q0size - 1;
                    let mut keyindexvar = Q0[Q0size];
                    let index = keyindexvar.index;
                    if sets0[index as usize].count == 0 {
                        continue
                    }
               
                    let hash = keyindexvar.hash;
                    let h1 = self.geth1(hash);
                    let h2 = self.geth2(hash);
                    stack[stacksize] = keyindexvar;
                    stacksize = stacksize + 1;
                    sets1[h1 as usize].xormask ^= hash;
                        
                    sets1[h1 as usize].count = sets1[h1 as usize].count.wrapping_sub(1); 
                    if sets1[h1 as usize].count == 1 {
                                    Q1[Q1size].index = h1;
                                    Q1[Q1size].hash = sets1[h1 as usize].xormask;
                                    Q1size = Q1size + 1;
                    }
                        
                    sets2[h2 as usize].xormask ^= hash;
                    sets2[h2 as usize].count = sets2[h2 as usize].count.wrapping_sub(1); 
                    if sets2[h2 as usize].count == 1 {
                                    Q2[Q2size].index = h2;
                                    Q2[Q2size].hash = sets2[h2 as usize].xormask;
                                    Q2size = Q2size + 1;
                            }
		    }
                     
                while Q1size > 0 {
                             
                    Q1size = Q1size - 1;
                    let mut keyindexvar  = Q1[Q1size];
                    let index = keyindexvar.index;

                    if sets1[index as usize].count == 0 {
                                    continue
                    }
                        
                    let hash = keyindexvar.hash;
                    let h0 = self.geth0(hash);
                    let h2 = self.geth2(hash);
                    keyindexvar.index += self.filter.BlockLength;
                    stack[stacksize] = keyindexvar;
                    stacksize = stacksize + 1;
                    sets0[h0 as usize].xormask ^= hash;
                    sets0[h0 as usize].count = sets0[h0 as usize].count.wrapping_sub(1);
                    if sets0[h0 as usize].count == 1 {
                                Q0[Q0size].index = h0;
                                Q0[Q0size].hash = sets0[h0 as usize].xormask;
                                Q0size = Q0size + 1;
		    }
                        
                    sets2[h2 as usize].xormask ^= hash;
                    sets2[h2 as usize].count = sets2[h2 as usize].count - 1;
                    
                    if sets2[h2 as usize].count == 1 {
                            Q2[Q2size].index = h2;
                            Q2[Q2size].hash = sets2[h2 as usize].xormask;
                            Q2size = Q2size + 1;
                    }
                }
                
                while Q2size > 0 {
                        Q2size = Q2size - 1;
                        let mut keyindexvar  = Q2[Q2size];
                        let index = keyindexvar.index;
                        if sets2[index as usize].count == 0 {
			    continue
			}
                        
                        let hash = keyindexvar.hash;
                        let h0 = self.geth0(hash);
                        let h1 = self.geth1(hash);
                        keyindexvar.index += (2*self.filter.BlockLength);
                        stack[stacksize] = keyindexvar;
                        stacksize = stacksize + 1;
                        
                        sets0[h0 as usize].xormask ^= hash; // need to bring this back 
                        sets0[h0 as usize].count = sets0[h0 as usize].count.wrapping_sub(1);
                        if sets0[h0 as usize].count == 1 {
					Q0[Q0size].index = h0;
					Q0[Q0size].hash = sets0[h0 as usize].xormask;
					Q0size = Q0size + 1;
			}
                        sets1[h1 as usize].xormask ^= hash;
                        sets1[h1 as usize].count = sets1[h1 as usize].count.wrapping_sub(1);
                        if sets1[h1 as usize].count == 1 {
                            Q1[Q1size].index = h1;
                            Q1[Q1size].hash = sets1[h1 as usize].xormask;
                            Q1size = Q1size + 1
                             }
                        
                        
                        }
                        
                    }
                       
                    if stacksize == size {
                        break;    
                    }
                    
                    for i in 0..sets0.len() {
                        sets0[i as usize] = xorset{xormask:0, count:0};
                    }
                    
                    for i in 0..sets1.len() {
			sets1[i as usize] = xorset{xormask: 0, count:0};
		    }
                    
                    for i in 0..sets2.len() {
			sets2[i as usize] = xorset{xormask: 0, count:0};
		    }
                    
                    self.filter.Seed = splitmix64(rngcounter);
                    
                }
                
        let mut stacksize = size;
        while stacksize > 0 {
            stacksize = stacksize - 1;
            let ki = stack[stacksize];
            let mut val = fingerprint(ki.hash) as u8;
            if ki.index < self.filter.BlockLength {
                val ^= self.filter.Fingerprints[(self.clone().geth1(ki.hash)+self.filter.BlockLength) as usize] ^ self.filter.Fingerprints[(self.clone().geth2(ki.hash)+2*self.filter.BlockLength) as usize]
            } else if ki.index < 2*self.filter.BlockLength {
                val ^= self.filter.Fingerprints[self.clone().geth0(ki.hash) as usize] ^ self.filter.Fingerprints[(self.clone().geth2(ki.hash)+2*self.filter.BlockLength) as usize]
            } else {
			val ^= self.filter.Fingerprints[self.clone().geth0(ki.hash) as usize] ^ self.filter.Fingerprints[(self.clone().geth1(ki.hash)+self.filter.BlockLength) as usize]
	    }
            self.filter.Fingerprints[ki.index as usize] = val;
        }
    }
    
    pub fn contains(&self, key: u64) -> bool {
        //unimplemented!();
        let mut hash = mixsplit(key, self.filter.Seed);
        let f = fingerprint(hash) as u8;
        let mut r0 = hash as u32;
        let mut r1 = rotl64(hash, 21) as u32;
        let mut r2 = rotl64(hash, 42) as u32; 
        let mut h0 = reduce(r0, self.filter.BlockLength);
        let mut h1 = reduce(r1, self.filter.BlockLength) + self.filter.BlockLength;
        let mut h2 = reduce(r2, self.filter.BlockLength) + 2*self.filter.BlockLength;
        
        f == (self.filter.Fingerprints[h0 as usize] ^ self.filter.Fingerprints[h1 as usize] ^
		self.filter.Fingerprints[h2 as usize])
        
    }
}

==> Xorfilter-0.0.7.crate: Xorfilter-0.0.7/src/lib.rs <==
#[derive(Debug, Default, Clone)]
pub struct Xor8 {
	Seed: u64,
	BlockLength: u32,
	Fingerprints: Vec<u8> 
}

/*
impl Default for Xor8 {
    fn default () -> Xor8 {
        Xor8{Seed: 0, BlockLength: 0, Fingerprints: Vec::new()}
    }
}
*/

#[derive(Clone, Debug)]
struct xorset {
	xormask: u64,
	count: u32
}

#[derive(Debug)]
struct hashes {
	h:  u64,
	h0: u32,
	h1: u32,
	h2: u32
}

#[derive(Copy, Clone, Debug)]
struct keyindex {
	hash: u64,
	index: u32 
}

pub fn murmur64(mut h: u64) -> u64 {
	h ^= h >> 33;
	h = h.wrapping_mul(0xff51afd7ed558ccd);
	h ^= h >> 33;
	h = h.wrapping_mul(0xc4ceb9fe1a85ec53);
	h ^= h >> 33;
	h
}

pub fn splitmix64(mut seed: u64) -> u64 {
    let mut z = seed.wrapping_add(0x9e3779b97f4a7c15); 
    z = (z ^ (z >> 30)).wrapping_mul(0xbf58476d1ce4e5b9); 
    z = (z ^ (z >> 27)).wrapping_mul(0x94d049bb133111eb);  
    z ^ (z >> 31)
}

pub fn mixsplit(key: u64, seed: u64) -> u64 {
    murmur64(key.wrapping_add(seed)) 
}

pub fn rotl64(n: u64, c: i64) -> u64 {
    (n << (c & 63)) | (n >> ((-c) & 63))
}

// http://lemire.me/blog/2016/06/27/a-fast-alternative-to-the-modulo-reduction/
pub fn reduce(hash: u32, n: u32) -> u32 {
        ((hash as u64).wrapping_mul(n as u64) >> 32) as u32
}

pub fn fingerprint(hash: u64) -> u64 {
	hash ^ (hash >> 32)
}

#[derive(Debug, Default, Clone)]
pub struct Xorfilter {
    filter: Xor8
}

impl Xorfilter {
    
    // TODO
    // type so this works
    // beyond just Xor8
    pub fn new() -> Xorfilter { 
        
               Xorfilter { 
                    filter: Xor8{Seed: 0, BlockLength: 0, Fingerprints: Vec::new()}
               }
               
    }

    fn geth0h1h2(&mut self, k: u64) -> hashes {
        let mut hash = mixsplit(k, self.filter.Seed);
        let mut r0 = hash as u32;
        let mut r1 = rotl64(hash, 21) as u32;
        let mut r2 = rotl64(hash, 42) as u32;
        
        let answer = hashes {
            h: hash,
            h0: reduce(r0, self.filter.BlockLength),
            h1: reduce(r1, self.filter.BlockLength),
            h2: reduce(r2, self.filter.BlockLength),
        };
        
        answer
    }
    
    fn geth0(&mut self, hash: u64) -> u32 {
        let r0 = hash as u32;
        reduce(r0, self.filter.BlockLength) 
    }
    
    fn geth1(&mut self, hash: u64) -> u32 {
        let r1 = rotl64(hash, 21) as u32;
        reduce(r1, self.filter.BlockLength)
    }
    
    fn geth2(&mut self, hash: u64) -> u32 {
        let r2 = rotl64(hash, 42) as u32;
        reduce(r2, self.filter.BlockLength)
    }
    
    pub fn populate(&mut self, keys: &Vec<u64>) { //-> Xor8 {
        let size = keys.len();
        let mut capacity = 32 + ((1.23*(size as f64)).ceil() as u32); //u32);
        capacity = capacity / 3 * 3; // round it down to a multiple of 3
        self.filter.BlockLength = capacity / 3; 
     
        self.filter.Fingerprints = vec![0; capacity as usize]; 
        let mut rngcounter: u64 = 1;
        self.filter.Seed = splitmix64(rngcounter);
        
        // unsure if defaulting values to zero will hurt performance
        // may need to change implementation in the future
        
        let mut Q0: Vec<keyindex> = vec![keyindex {hash: 0, index: 0}; self.filter.BlockLength as usize]; 
        let mut Q1: Vec<keyindex> = vec![keyindex {hash: 0, index: 0}; self.filter.BlockLength as usize]; 
        let mut Q2: Vec<keyindex> = vec![keyindex {hash: 0, index: 0}; self.filter.BlockLength as usize]; 
        
        let mut stack: Vec<keyindex> = vec![keyindex {hash: 0, index: 0}; size]; 
        let mut sets0: Vec<xorset> = vec![xorset {xormask:0, count:0}; self.filter.BlockLength as usize]; 
        let mut sets1: Vec<xorset> = vec![xorset {xormask:0, count:0}; self.filter.BlockLength as usize]; 
        let mut sets2: Vec<xorset> = vec![xorset {xormask:0, count:0}; self.filter.BlockLength as usize]; 
        
        while true {
            for i in 0..size {
                
                let key = keys[i];
                let hs = self.geth0h1h2(key);
                sets0[hs.h0 as usize].xormask ^= hs.h;
                sets0[hs.h0 as usize].count = sets0[hs.h0 as usize].count + 1;
                sets1[hs.h1 as usize].xormask ^= hs.h;
                sets1[hs.h1 as usize].count = sets1[hs.h1 as usize].count + 1;
                sets2[hs.h2 as usize].xormask ^= hs.h;
                sets2[hs.h2 as usize].count = sets2[hs.h2 as usize].count + 1;
                
	    }
                
            
            let mut Q0size = 0;
            let mut Q1size = 0;
            let mut Q2size = 0;
           
            for i in 0u32..self.filter.BlockLength {
                if sets0[i as usize].count == 1 {
                            Q0[Q0size].index = i;
                            Q0[Q0size].hash = sets0[i as usize].xormask;
                            Q0size = Q0size + 1;
                        }
            }
            
            for i in 0u32..self.filter.BlockLength {
			if sets1[i as usize].count == 1 {
				Q1[Q1size].index = i;
				Q1[Q1size].hash = sets1[i as usize].xormask;
				Q1size = Q1size + 1;
			}
		}
		
            for i in 0u32..self.filter.BlockLength {
			if sets2[i as usize].count == 1 {
				Q2[Q2size].index = i;
				Q2[Q2size].hash = sets2[i as usize].xormask;
				Q2size = Q2size + 1;
			}
		}
                
            let mut stacksize = 0;
               
            while Q0size+Q1size+Q2size > 0 {
                while Q0size > 0 {
                    Q0size = Q0size - 1;
                    let mut keyindexvar = Q0[Q0size];
                    let index = keyindexvar.index;
                    if sets0[index as usize].count == 0 {
                        continue
                    }
               
                    let hash = keyindexvar.hash;
                    let h1 = self.geth1(hash);
                    let h2 = self.geth2(hash);
                    stack[stacksize] = keyindexvar;
                    stacksize = stacksize + 1;
                    sets1[h1 as usize].xormask ^= hash;
                        
                    sets1[h1 as usize].count = sets1[h1 as usize].count.wrapping_sub(1); 
                    if sets1[h1 as usize].count == 1 {
                                    Q1[Q1size].index = h1;
                                    Q1[Q1size].hash = sets1[h1 as usize].xormask;
                                    Q1size = Q1size + 1;
                    }
                        
                    sets2[h2 as usize].xormask ^= hash;
                    sets2[h2 as usize].count = sets2[h2 as usize].count.wrapping_sub(1); 
                    if sets2[h2 as usize].count == 1 {
                                    Q2[Q2size].index = h2;
                                    Q2[Q2size].hash = sets2[h2 as usize].xormask;
                                    Q2size = Q2size + 1;
                            }
		    }
                     
                while Q1size > 0 {
                             
                    Q1size = Q1size - 1;
                    let mut keyindexvar  = Q1[Q1size];
                    let index = keyindexvar.index;

                    if sets1[index as usize].count == 0 {
                                    continue
                    }
                        
                    let hash = keyindexvar.hash;
                    let h0 = self.geth0(hash);
                    let h2 = self.geth2(hash);
                    keyindexvar.index += self.filter.BlockLength;
                    stack[stacksize] = keyindexvar;
                    stacksize = stacksize + 1;
                    sets0[h0 as usize].xormask ^= hash;
                    sets0[h0 as usize].count = sets0[h0 as usize].count.wrapping_sub(1);
                    if sets0[h0 as usize].count == 1 {
                                Q0[Q0size].index = h0;
                                Q0[Q0size].hash = sets0[h0 as usize].xormask;
                                Q0size = Q0size + 1;
		    }
                        
                    sets2[h2 as usize].xormask ^= hash;
                    sets2[h2 as usize].count = sets2[h2 as usize].count - 1;
                    
                    if sets2[h2 as usize].count == 1 {
                            Q2[Q2size].index = h2;
                            Q2[Q2size].hash = sets2[h2 as usize].xormask;
                            Q2size = Q2size + 1;
                    }
                }
                
                while Q2size > 0 {
                        Q2size = Q2size - 1;
                        let mut keyindexvar  = Q2[Q2size];
                        let index = keyindexvar.index;
                        if sets2[index as usize].count == 0 {
			    continue
			}
                        
                        let hash = keyindexvar.hash;
                        let h0 = self.geth0(hash);
                        let h1 = self.geth1(hash);
                        keyindexvar.index += (2*self.filter.BlockLength);
                        stack[stacksize] = keyindexvar;
                        stacksize = stacksize + 1;
                        
                        sets0[h0 as usize].xormask ^= hash; // need to bring this back 
                        sets0[h0 as usize].count = sets0[h0 as usize].count.wrapping_sub(1);
                        if sets0[h0 as usize].count == 1 {
					Q0[Q0size].index = h0;
					Q0[Q0size].hash = sets0[h0 as usize].xormask;
					Q0size = Q0size + 1;
			}
                        sets1[h1 as usize].xormask ^= hash;
                        sets1[h1 as usize].count = sets1[h1 as usize].count.wrapping_sub(1);
                        if sets1[h1 as usize].count == 1 {
                            Q1[Q1size].index = h1;
                            Q1[Q1size].hash = sets1[h1 as usize].xormask;
                            Q1size = Q1size + 1
                             }
                        
                        
                        }
                        
                    }
                       
                    if stacksize == size {
                        break;    
                    }
                    
                    for i in 0..sets0.len() {
                        sets0[i as usize] = xorset{xormask:0, count:0};
                    }
                    
                    for i in 0..sets1.len() {
			sets1[i as usize] = xorset{xormask: 0, count:0};
		    }
                    
                    for i in 0..sets2.len() {
			sets2[i as usize] = xorset{xormask: 0, count:0};
		    }
                    
                    self.filter.Seed = splitmix64(rngcounter);
                    
                }
                
        let mut stacksize = size;
        while stacksize > 0 {
            stacksize = stacksize - 1;
            let ki = stack[stacksize];
            let mut val = fingerprint(ki.hash) as u8;
            if ki.index < self.filter.BlockLength {
                val ^= self.filter.Fingerprints[(self.clone().geth1(ki.hash)+self.filter.BlockLength) as usize] ^ self.filter.Fingerprints[(self.clone().geth2(ki.hash)+2*self.filter.BlockLength) as usize]
            } else if ki.index < 2*self.filter.BlockLength {
                val ^= self.filter.Fingerprints[self.clone().geth0(ki.hash) as usize] ^ self.filter.Fingerprints[(self.clone().geth2(ki.hash)+2*self.filter.BlockLength) as usize]
            } else {
			val ^= self.filter.Fingerprints[self.clone().geth0(ki.hash) as usize] ^ self.filter.Fingerprints[(self.clone().geth1(ki.hash)+self.filter.BlockLength) as usize]
	    }
            self.filter.Fingerprints[ki.index as usize] = val;
        }
    }
    
    pub fn contains(&self, key: u64) -> bool {
        let mut hash = mixsplit(key, self.filter.Seed);
        let f = fingerprint(hash) as u8;
        let mut r0 = hash as u32;
        let mut r1 = rotl64(hash, 21) as u32;
        let mut r2 = rotl64(hash, 42) as u32; 
        let mut h0 = reduce(r0, self.filter.BlockLength);
        let mut h1 = reduce(r1, self.filter.BlockLength) + self.filter.BlockLength;
        let mut h2 = reduce(r2, self.filter.BlockLength) + 2*self.filter.BlockLength;
        
        f == (self.filter.Fingerprints[h0 as usize] ^ self.filter.Fingerprints[h1 as usize] ^
		self.filter.Fingerprints[h2 as usize])
        
    }
}
